# vim: set et ts=4 sw=4 fileencoding=utf-8:
'''
yalp.test
=========
'''
from yalp.test.testcase import YalpTestCase
from yalp.test.utils import override_settings


__all__ = [
    'YalpTestCase', 'override_settings'
]
